@@ -1,5 +1,7 @@
 # -*- coding: utf-8 -*-
 
+from __future__ import division
+
 __copyright__ = "Copyright (C) 2009 Andreas Kloeckner"
 
 __license__ = """
@@ -176,6 +178,8 @@ class Program(object):
     def build(self, options=[], devices=None, cache_dir=None):
         if isinstance(options, str):
             options = [options]
+        elif isinstance(options, unicode):
+            options = [options.encode("utf8")]
 
         options = (options
                 + _DEFAULT_BUILD_OPTIONS
@@ -281,26 +285,16 @@ def link_program(context, programs, options=[], devices=None):
 
 def _add_functionality():
     cls_to_info_cls = {
-            _cl.Platform:
-                (_cl.Platform.get_info, _cl.platform_info),
-            _cl.Device:
-                (_cl.Device.get_info, _cl.device_info),
-            _cl.Context:
-                (_cl.Context.get_info, _cl.context_info),
-            _cl.CommandQueue:
-                (_cl.CommandQueue.get_info, _cl.command_queue_info),
-            _cl.Event:
-                (_cl.Event.get_info, _cl.event_info),
-            _cl.MemoryObjectHolder:
-                (MemoryObjectHolder.get_info, _cl.mem_info),
-            Image:
-                (_cl.Image.get_image_info, _cl.image_info),
-            Program:
-                (Program.get_info, _cl.program_info),
-            Kernel:
-                (Kernel.get_info, _cl.kernel_info),
-            _cl.Sampler:
-                (Sampler.get_info, _cl.sampler_info),
+            _cl.Platform: (_cl.Platform.get_info, _cl.platform_info),
+            _cl.Device: (_cl.Device.get_info, _cl.device_info),
+            _cl.Context: (_cl.Context.get_info, _cl.context_info),
+            _cl.CommandQueue: (_cl.CommandQueue.get_info, _cl.command_queue_info),
+            _cl.Event: (_cl.Event.get_info, _cl.event_info),
+            _cl.MemoryObjectHolder: (MemoryObjectHolder.get_info, _cl.mem_info),
+            Image: (_cl.Image.get_image_info, _cl.image_info),
+            Program: (Program.get_info, _cl.program_info),
+            Kernel: (Kernel.get_info, _cl.kernel_info),
+            _cl.Sampler: (Sampler.get_info, _cl.sampler_info),
             }
 
     def to_string(cls, value, default_format=None):
@@ -340,7 +334,19 @@ def _add_functionality():
     def platform_repr(self):
         return "<pyopencl.Platform '%s' at 0x%x>" % (self.name, self.int_ptr)
 
+    def platform_get_cl_version(self):
+        import re
+        version_string = self.version
+        match = re.match(r"^OpenCL ([0-9]+)\.([0-9]+) .*$", version_string)
+        if match is None:
+            raise RuntimeError("platform %s returned non-conformant "
+                               "platform version string '%s'" %
+                               (self, version_string))
+
+        return int(match.group(1)), int(match.group(2))
+
     Platform.__repr__ = platform_repr
+    Platform._get_cl_version = platform_get_cl_version
 
     # }}}
 
@@ -367,16 +373,7 @@ def _add_functionality():
                 ", ".join(repr(dev) for dev in self.devices))
 
     def context_get_cl_version(self):
-        import re
-        platform = self.devices[0].platform
-        plat_version_string = platform.version
-        match = re.match(r"^OpenCL ([0-9]+)\.([0-9]+) .*$",
-                plat_version_string)
-        if match is None:
-            raise RuntimeError("platform %s returned non-conformant "
-                    "platform version string '%s'" % (platform, plat_version_string))
-
-        return int(match.group(1)), int(match.group(2))
+        return self.devices[0].platform._get_cl_version()
 
     Context.__repr__ = context_repr
     from pytools import memoize_method
@@ -810,8 +807,6 @@ def create_some_context(interactive=None, answers=None):
 
     if not platforms:
         raise Error("no platforms found")
-    elif len(platforms) == 1:
-        platform, = platforms
     else:
         if not answers:
             cc_print("Choose platform:")
@@ -946,6 +941,15 @@ def enqueue_copy(queue, dest, src, **kwargs):
     :return: A :class:`NannyEvent` if the transfer involved a
         host-side buffer, otherwise an :class:`Event`.
 
+    .. note::
+
+        Two types of 'buffer' occur in the arguments to this function,
+        :class:`Buffer` and 'host-side buffers'. The latter are
+        defined by Python and commonly called `buffer objects
+        <https://docs.python.org/3.4/c-api/buffer.html>`_.
+        Make sure to always be clear on whether a :class:`Buffer` or a
+        Python buffer object is needed.
+
     .. ------------------------------------------------------------------------
     .. rubric :: Transfer :class:`Buffer` ↔ host
     .. ------------------------------------------------------------------------
@@ -1208,10 +1212,7 @@ def enqueue_fill_buffer(queue, mem, pattern, offset, size, wait_for=None):
         from warnings import warn
         warn("The context for this queue does not declare OpenCL 1.2 support, so "
                 "the next thing you might see is a crash")
-    return _cl._enqueue_fill_buffer(queue, mem, pattern, offset,
-            size, wait_for=None)
-
-
+    return _cl._enqueue_fill_buffer(queue, mem, pattern, offset, size, wait_for)
 
 # }}}
 
