@@ -783,6 +783,7 @@ _IGNORED_WORDS = set("""
         pragma __attribute__ __global __kernel __local
         get_local_size get_local_id cl_khr_fp64 reqd_work_group_size
         get_num_groups barrier get_group_id
+        CL_VERSION_1_1 __OPENCL_C_VERSION__ 120
 
         _final_update _debug_scan kernel_name
 
@@ -1101,9 +1102,9 @@ class GenericScanKernel(_GenericScanKernelBase):
 
                 if have_sol_above_floor:
                     # delete all solutions not meeting the wg size floor
-                    solutions = [(total, k_group_size, wg_size)
-                            for total, k_group_size, wg_size in solutions
-                            if wg_size >= wg_size_floor]
+                    solutions = [(total, try_k_group_size, try_wg_size)
+                            for total, try_k_group_size, try_wg_size in solutions
+                            if try_wg_size >= wg_size_floor]
                     break
 
         _, k_group_size, max_scan_wg_size = max(solutions)
